import os
import ast
import astunparse

class Creator:
    def __init__(self, directory, drf_handlers, template_handler):
        self.directory = directory
        self.drf_handlers = drf_handlers
        self.template_handler = template_handler
        
    def create(self):
        model_classes = self.template_handler.get_model_classes(self.directory)

        templates = dict()
        for handler in self.drf_handlers:
            templates[handler.code] = self.template_handler.get_template(handler.code)

        for model_class in model_classes:
            for handler in self.drf_handlers:
                handler.handle(templates.get(handler.code), model_class)

        for handler in self.drf_handlers:
            self.template_handler.write_result(handler.result_name, templates.get(handler.code))


class TemplateHandler:
    def __init__(self, file_handler):
        super().__init__()
        self.file_handler = file_handler
        
    def get_model_classes(self, directory):
        path  = os.path.join(directory, 'models.py')
        models_ast = ast.parse(self.file_handler.read(path))

        return [clss for clss in models_ast.body if clss.__class__ is ast.ClassDef]

    def get_template(self, template_name, **kwargs):
        script_folder = os.path.dirname(os.path.abspath(__file__))
        path = os.path.join(script_folder, f'templates/{template_name}')

        return ast.parse(self.file_handler.read(path, **kwargs))

    def write_result(self, result_file_name, template):
        self.file_handler.write(result_file_name, astunparse.unparse(template))


class FileHandler:
    def read(self, path, **kwargs):
        with open(path, 'r') as file:
            return file.read().format(**kwargs)

    def write(self, path, content):
         with open(path, 'w') as wfile:
            wfile.write(content) 

    


from .core import Creator, TemplateHandler, FileHandler
from .drf_handlers import SerializersHandler, ViewsHandler, URLsHandler
    
def create_all(current_directory):
    response = input("""Warning: the content of serializers.py, views.py and urls.py will be overwritten
Do you want to continue? [Y/n] """)

    if response == "" or response == 'y' or response == 'Y':
        creator = _get_creator(current_directory, SerializersHandler, ViewsHandler, URLsHandler)

        creator.create()

def create_serializers(current_directory):
    creator = _get_creator(current_directory, SerializersHandler)

    creator.create()

def create_views(current_directory):
    creator = _get_creator(current_directory, ViewsHandler)

    creator.create()

def create_urls(current_directory):
    creator = _get_creator(current_directory, URLsHandler)

    creator.create()
    

def _get_creator(directory, *args):
    template_handler = TemplateHandler(FileHandler())

    return Creator(directory, [klass(template_handler) for klass in args], template_handler)



import ast
import re

class DRFHandler:
    def __init__(self, template_handler):
        super().__init__()
        self.template_handler = template_handler

    @property
    def code(self):
        raise NotImplementedError('code must be implemented by subclass')

    @property
    def result_name(self):
        raise NotImplementedError('result_name must be implemented by subclass')
    
    def handle(self, template, model_class):
        raise NotImplementedError('handle must be implemented by subclass')


class SerializersHandler(DRFHandler):
    @property
    def code(self):
        return 'serializers'

    @property
    def result_name(self):
        return 'serializers.py'

    def handle(self, template, model_class):
        class_fields = [field for field in model_class.body if field.__class__ is ast.Assign]

        template.body.append(self.template_handler.get_template('serializer', model_class_name=model_class.name, 
            field_names=", ".join([f"'{field_name.targets[0].id}'" for field_name in class_fields])))



class ViewsHandler(DRFHandler):
    @property
    def code(self):
        return 'views'

    @property
    def result_name(self):
        return 'views.py'

    def handle(self, template, model_class):
        template.body.append(self.template_handler.get_template('viewset', model_class_name=model_class.name))


class URLsHandler(DRFHandler):
    @property
    def code(self):
        return 'urls'

    @property
    def result_name(self):
        return 'urls.py'

    def handle(self, template, model_class):
        pattern = re.compile(r'(?<!^)(?=[A-Z])')
        mc_name_snake = pattern.sub('-', model_class.name).lower()
        template.body.insert(len(template.body)-1, self.template_handler.get_template('url', model_class_name=model_class.name, model_class_name_snake=mc_name_snake))


